@@ -2,6 +2,9 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+Components.utils.import("resource://gre/modules/Services.jsm");
+Components.utils.import("resource://gre/modules/Preferences.jsm");
+
 var gStartDate = null;
 var gEndDate = null;
 var gStartTimezone = null;
@@ -25,11 +28,15 @@ function onLoad() {
     window.addEventListener("resize", onResize, true);
     window.addEventListener("modify", onModify, true);
     window.addEventListener("rowchange", onRowChange, true);
-    window.addEventListener("DOMMouseScroll", onMouseScroll, true);
     window.addEventListener("DOMAttrModified", onAttrModified, true);
     window.addEventListener("timebar", onTimebar, true);
     window.addEventListener("timechange", onTimeChange, true);
 
+    // As long as DOMMouseScroll is still implemented, we need to keep it
+    // around to make sure scrolling is blocked.
+    window.addEventListener("wheel", onMouseScroll, true);
+    window.addEventListener("DOMMouseScroll", onMouseScroll, true);
+
     var args = window.arguments[0];
     var startTime = args.startTime;
     var endTime = args.endTime;
@@ -46,6 +53,10 @@ function onLoad() {
 
     // Make sure zoom factor is set up correctly (from persisted value)
     setZoomFactor(zoom.value);
+    if(gZoomFactor == 100) {
+        // if zoom factor was not changed, make sure it is applied at least once
+        applyCurrentZoomFactor();
+    }
 
     initTimeRange();
 
@@ -68,7 +79,8 @@ function onLoad() {
 
     loadDateTime(startTime, endTime);
     propagateDateTime();
-
+    // Set the scroll bar at where the event is
+    scrollToCurrentTime();
     updateButtons();
 
     // we need to enforce several layout constraints which can't be modelled
@@ -103,12 +115,10 @@ function onLoad() {
             }
         }
     }
-    var pb2 = Components.classes["@mozilla.org/preferences-service;1"].
-              getService(Components.interfaces.nsIPrefBranch);
-    pb2.addObserver("calendar.", prefObserver, false);
+    Services.prefs.addObserver("calendar.", prefObserver, false);
     window.addEventListener("unload",
         function() {
-            pb2.removeObserver("calendar.", prefObserver);
+            Services.prefs.removeObserver("calendar.", prefObserver);
         },
         false);
 
@@ -212,7 +222,9 @@ function propagateDateTime() {
     var startTime = gStartDate.getInTimezone(kDefaultTimezone);
     var endTime = gEndDate.getInTimezone(kDefaultTimezone);
     if ((startTime.hour < gStartHour) ||
-        (endTime.hour > gEndHour) ||
+        (startTime.hour >= gEndHour) ||
+        (endTime.hour >= gEndHour) ||
+        (startTime.day != endTime.day) ||
         (startTime.isDate)) {
         setForce24Hours(true);
     }
@@ -331,23 +343,24 @@ function updateStartTime() {
         return;
     }
 
-    var startWidgetId = "event-starttime";
-    var endWidgetId = "event-endtime";
+    let startWidgetId = "event-starttime";
+    let endWidgetId = "event-endtime";
 
-    var startWidget = document.getElementById(startWidgetId);
-    var endWidget = document.getElementById(endWidgetId);
+    let startWidget = document.getElementById(startWidgetId);
+    let endWidget = document.getElementById(endWidgetId);
 
     // jsDate is always in OS timezone, thus we create a calIDateTime
     // object from the jsDate representation and simply set the new
     // timezone instead of converting.
-    var start = jsDateToDateTime(startWidget.value,
-                                 gDisplayTimezone ? gStartTimezone : calendarDefaultTimezone());
+    let timezone = gDisplayTimezone ? gStartTimezone : calendarDefaultTimezone();
+    let start = cal.jsDateToDateTime(startWidget.value, timezone);
+                                    
     gStartDate = start.clone();
     start.addDuration(gDuration);
     gEndDate = start.getInTimezone(gEndTimezone);
 
-    var allDayElement = document.getElementById("all-day");
-    var allDay = allDayElement.getAttribute("checked") == "true";
+    let allDayElement = document.getElementById("all-day");
+    let allDay = allDayElement.getAttribute("checked") == "true";
     if (allDay) {
         gStartDate.isDate = true;
         gEndDate.isDate = true;
@@ -374,7 +387,7 @@ function updateEndTime() {
     var saveEndTime = gEndDate;
     var kDefaultTimezone = calendarDefaultTimezone();
 
-    gStartDate = jsDateToDateTime(startWidget.value,
+    gStartDate = cal.jsDateToDateTime(startWidget.value,
                                   gDisplayTimezone ? gStartTimezone : calendarDefaultTimezone());
 
     var timezone = gEndTimezone;
@@ -383,8 +396,8 @@ function updateEndTime() {
         !compareObjects(gStartTimezone, gEndTimezone)) {
         timezone = gStartTimezone;
     }
-    gEndDate = jsDateToDateTime(endWidget.value,
-                                gDisplayTimezone ? timezone : kDefaultTimezone);
+    gEndDate = cal.jsDateToDateTime(endWidget.value,
+                                    gDisplayTimezone ? timezone : kDefaultTimezone);
 
     var allDayElement = document.getElementById("all-day");
     var allDay = allDayElement.getAttribute("checked") == "true";
@@ -408,12 +421,7 @@ function updateEndTime() {
 
     if (warning) {
         var callback = function() {
-            var promptService =
-                Components.classes[
-                    "@mozilla.org/embedcomp/prompt-service;1"]
-                    .getService(
-                        Components.interfaces.nsIPromptService);
-            promptService.alert(
+            Services.prompt.alert(
                 null,
                 document.title,
                 calGetString("calendar", "warningEndBeforeStart"));
@@ -615,8 +623,9 @@ function onChangeCalendar(calendar) {
     // assume we're the organizer [in case that the calendar
     // does not support the concept of identities].
     gIsInvitation = false;
-    if (calInstanceOf(args.item.calendar, Components.interfaces.calISchedulingSupport)) {
-        gIsInvitation = args.item.calendar.isInvitation(args.item);
+    calendar = cal.wrapInstance(args.item.calendar, Components.interfaces.calISchedulingSupport);
+    if (calendar) {
+        gIsInvitation = calendar.isInvitation(args.item);
     }
 
     if (gIsReadOnly || gIsInvitation) {
@@ -719,7 +728,7 @@ function onPreviousSlot() {
  */
 function scrollToCurrentTime() {
     var timebar = document.getElementById("timebar");
-    var ratio = (gStartDate.hour - gStartHour) * timebar.step;
+    var ratio = (gStartDate.hour - gStartHour - 1) * timebar.step;
     if (ratio <= 0.0) {
         ratio = 0.0;
     }
@@ -747,6 +756,14 @@ function setZoomFactor(aValue) {
     }
 
     gZoomFactor = aValue;
+    applyCurrentZoomFactor();
+    return aValue;
+}
+
+/**
+ * applies the current zoom factor for the time grid
+ */
+function applyCurrentZoomFactor() {
     var timebar = document.getElementById("timebar");
     timebar.zoomFactor = gZoomFactor;
     var selectionbar = document.getElementById("selection-bar");
@@ -768,8 +785,6 @@ function setZoomFactor(aValue) {
         grid.scroll = ratio;
         selectionbar.ratio = ratio;
     }
-
-    return aValue;
 }
 
 /**
@@ -820,8 +835,8 @@ function initTimeRange() {
         gStartHour = 0;
         gEndHour = 24;
     } else {
-        gStartHour = getPrefSafe("calendar.view.daystarthour", 8);
-        gEndHour = getPrefSafe("calendar.view.dayendhour", 19);
+        gStartHour = Preferences.get("calendar.view.daystarthour", 8);
+        gEndHour = Preferences.get("calendar.view.dayendhour", 19);
     }
 }
 
@@ -856,7 +871,7 @@ function onRowChange(event) {
 /**
  * Handler function to take care of mouse scrolling on the window
  *
- * @param event     The DOMMouseScroll event caused by scrolling.
+ * @param event     The wheel event caused by scrolling.
  */
 function onMouseScroll(event) {
     // ignore mouse scrolling for now...
