@@ -9,7 +9,6 @@
 #include "nsError.h"
 #include "SVGAnimatedLengthList.h"
 #include "nsCOMPtr.h"
-#include "nsContentUtils.h"
 #include "mozilla/dom/SVGLengthListBinding.h"
 #include <algorithm>
 
@@ -36,10 +35,12 @@ void UpdateListIndicesFromIndex(FallibleTArray<DOMSVGLength*>& aItemsArray,
 
 namespace mozilla {
 
-// We could use NS_IMPL_CYCLE_COLLECTION_1, except that in Unlink() we need to
+// We could use NS_IMPL_CYCLE_COLLECTION(, except that in Unlink() we need to
 // clear our DOMSVGAnimatedLengthList's weak ref to us to be safe. (The other
 // option would be to not unlink and rely on the breaking of the other edges in
 // the cycle, as NS_SVG_VAL_IMPL_CYCLE_COLLECTION does.)
+NS_IMPL_CYCLE_COLLECTION_CLASS(DOMSVGLengthList)
+
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(DOMSVGLengthList)
   if (tmp->mAList) {
     if (tmp->IsAnimValList()) {
@@ -68,11 +69,42 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(DOMSVGLengthList)
 NS_INTERFACE_MAP_END
 
 JSObject*
-DOMSVGLengthList::WrapObject(JSContext *cx, JS::Handle<JSObject*> scope)
+DOMSVGLengthList::WrapObject(JSContext *cx)
 {
-  return mozilla::dom::SVGLengthListBinding::Wrap(cx, scope, this);
+  return mozilla::dom::SVGLengthListBinding::Wrap(cx, this);
 }
 
+//----------------------------------------------------------------------
+// Helper class: AutoChangeLengthListNotifier
+// Stack-based helper class to pair calls to WillChangeLengthList and
+// DidChangeLengthList.
+class MOZ_STACK_CLASS AutoChangeLengthListNotifier
+{
+public:
+  AutoChangeLengthListNotifier(DOMSVGLengthList* aLengthList MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
+    : mLengthList(aLengthList)
+  {
+    MOZ_GUARD_OBJECT_NOTIFIER_INIT;
+    MOZ_ASSERT(mLengthList, "Expecting non-null lengthList");
+    mEmptyOrOldValue =
+      mLengthList->Element()->WillChangeLengthList(mLengthList->AttrEnum());
+  }
+
+  ~AutoChangeLengthListNotifier()
+  {
+    mLengthList->Element()->DidChangeLengthList(mLengthList->AttrEnum(),
+                                                mEmptyOrOldValue);
+    if (mLengthList->IsAnimating()) {
+      mLengthList->Element()->AnimationNeedsResample();
+    }
+  }
+
+private:
+  DOMSVGLengthList* const mLengthList;
+  nsAttrValue       mEmptyOrOldValue;
+  MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
+};
+
 void
 DOMSVGLengthList::InternalListLengthWillChange(uint32_t aNewLength)
 {
@@ -129,7 +161,7 @@ DOMSVGLengthList::Clear(ErrorResult& aError)
   }
 
   if (LengthNoFlush() > 0) {
-    nsAttrValue emptyOrOldValue = Element()->WillChangeLengthList(AttrEnum());
+    AutoChangeLengthListNotifier notifier(this);
     // Notify any existing DOM items of removal *before* truncating the lists
     // so that they can find their SVGLength internal counterparts and copy
     // their values. This also notifies the animVal list:
@@ -137,15 +169,11 @@ DOMSVGLengthList::Clear(ErrorResult& aError)
 
     mItems.Clear();
     InternalList().Clear();
-    Element()->DidChangeLengthList(AttrEnum(), emptyOrOldValue);
-    if (mAList->IsAnimating()) {
-      Element()->AnimationNeedsResample();
-    }
   }
 }
 
-already_AddRefed<nsIDOMSVGLength>
-DOMSVGLengthList::Initialize(nsIDOMSVGLength *newItem,
+already_AddRefed<DOMSVGLength>
+DOMSVGLengthList::Initialize(DOMSVGLength& newItem,
                              ErrorResult& error)
 {
   if (IsAnimValList()) {
@@ -153,30 +181,41 @@ DOMSVGLengthList::Initialize(nsIDOMSVGLength *newItem,
     return nullptr;
   }
 
-  // If newItem is already in a list we should insert a clone of newItem, and
-  // for consistency, this should happen even if *this* is the list that
-  // newItem is currently in. Note that in the case of newItem being in this
-  // list, the Clear() call before the InsertItemBefore() call would remove it
-  // from this list, and so the InsertItemBefore() call would not insert a
-  // clone of newItem, it would actually insert newItem. To prevent that from
-  // happening we have to do the clone here, if necessary.
+  // If newItem already has an owner or is reflecting an attribute, we should
+  // insert a clone of newItem, and for consistency, this should happen even if
+  // *this* is the list that newItem is currently in. Note that in the case of
+  // newItem being in this list, the Clear() call before the InsertItemBefore()
+  // call would remove it from this list, and so the InsertItemBefore() call
+  // would not insert a clone of newItem, it would actually insert newItem. To
+  // prevent that from happening we have to do the clone here, if necessary.
 
-  nsCOMPtr<DOMSVGLength> domItem = do_QueryInterface(newItem);
+  nsRefPtr<DOMSVGLength> domItem = &newItem;
   if (!domItem) {
     error.Throw(NS_ERROR_DOM_SVG_WRONG_TYPE_ERR);
     return nullptr;
   }
-  if (domItem->HasOwner()) {
-    newItem = domItem->Copy();
+  if (domItem->HasOwner() || domItem->IsReflectingAttribute()) {
+    domItem = domItem->Copy();
   }
 
   ErrorResult rv;
   Clear(rv);
   MOZ_ASSERT(!rv.Failed());
-  return InsertItemBefore(newItem, 0, error);
+  return InsertItemBefore(*domItem, 0, error);
 }
 
-nsIDOMSVGLength*
+already_AddRefed<DOMSVGLength>
+DOMSVGLengthList::GetItem(uint32_t index, ErrorResult& error)
+{
+  bool found;
+  nsRefPtr<DOMSVGLength> item = IndexedGetter(index, found, error);
+  if (!found) {
+    error.Throw(NS_ERROR_DOM_INDEX_SIZE_ERR);
+  }
+  return item.forget();
+}
+
+already_AddRefed<DOMSVGLength>
 DOMSVGLengthList::IndexedGetter(uint32_t index, bool& found, ErrorResult& error)
 {
   if (IsAnimValList()) {
@@ -184,14 +223,13 @@ DOMSVGLengthList::IndexedGetter(uint32_t index, bool& found, ErrorResult& error)
   }
   found = index < LengthNoFlush();
   if (found) {
-    EnsureItemAt(index);
-    return mItems[index];
+    return GetItemAt(index);
   }
   return nullptr;
 }
 
-already_AddRefed<nsIDOMSVGLength>
-DOMSVGLengthList::InsertItemBefore(nsIDOMSVGLength *newItem,
+already_AddRefed<DOMSVGLength>
+DOMSVGLengthList::InsertItemBefore(DOMSVGLength& newItem,
                                    uint32_t index,
                                    ErrorResult& error)
 {
@@ -206,12 +244,12 @@ DOMSVGLengthList::InsertItemBefore(nsIDOMSVGLength *newItem,
     return nullptr;
   }
 
-  nsCOMPtr<DOMSVGLength> domItem = do_QueryInterface(newItem);
+  nsRefPtr<DOMSVGLength> domItem = &newItem;
   if (!domItem) {
     error.Throw(NS_ERROR_DOM_SVG_WRONG_TYPE_ERR);
     return nullptr;
   }
-  if (domItem->HasOwner()) {
+  if (domItem->HasOwner() || domItem->IsReflectingAttribute()) {
     domItem = domItem->Copy(); // must do this before changing anything!
   }
 
@@ -222,7 +260,7 @@ DOMSVGLengthList::InsertItemBefore(nsIDOMSVGLength *newItem,
     return nullptr;
   }
 
-  nsAttrValue emptyOrOldValue = Element()->WillChangeLengthList(AttrEnum());
+  AutoChangeLengthListNotifier notifier(this);
   // Now that we know we're inserting, keep animVal list in sync as necessary.
   MaybeInsertNullInAnimValListAt(index);
 
@@ -236,15 +274,11 @@ DOMSVGLengthList::InsertItemBefore(nsIDOMSVGLength *newItem,
 
   UpdateListIndicesFromIndex(mItems, index + 1);
 
-  Element()->DidChangeLengthList(AttrEnum(), emptyOrOldValue);
-  if (mAList->IsAnimating()) {
-    Element()->AnimationNeedsResample();
-  }
   return domItem.forget();
 }
 
-already_AddRefed<nsIDOMSVGLength>
-DOMSVGLengthList::ReplaceItem(nsIDOMSVGLength *newItem,
+already_AddRefed<DOMSVGLength>
+DOMSVGLengthList::ReplaceItem(DOMSVGLength& newItem,
                               uint32_t index,
                               ErrorResult& error)
 {
@@ -253,7 +287,7 @@ DOMSVGLengthList::ReplaceItem(nsIDOMSVGLength *newItem,
     return nullptr;
   }
 
-  nsCOMPtr<DOMSVGLength> domItem = do_QueryInterface(newItem);
+  nsRefPtr<DOMSVGLength> domItem = &newItem;
   if (!domItem) {
     error.Throw(NS_ERROR_DOM_SVG_WRONG_TYPE_ERR);
     return nullptr;
@@ -262,11 +296,11 @@ DOMSVGLengthList::ReplaceItem(nsIDOMSVGLength *newItem,
     error.Throw(NS_ERROR_DOM_INDEX_SIZE_ERR);
     return nullptr;
   }
-  if (domItem->HasOwner()) {
+  if (domItem->HasOwner() || domItem->IsReflectingAttribute()) {
     domItem = domItem->Copy(); // must do this before changing anything!
   }
 
-  nsAttrValue emptyOrOldValue = Element()->WillChangeLengthList(AttrEnum());
+  AutoChangeLengthListNotifier notifier(this);
   if (mItems[index]) {
     // Notify any existing DOM item of removal *before* modifying the lists so
     // that the DOM item can copy the *old* value at its index:
@@ -280,14 +314,10 @@ DOMSVGLengthList::ReplaceItem(nsIDOMSVGLength *newItem,
   // would end up reading bad data from InternalList()!
   domItem->InsertingIntoList(this, AttrEnum(), index, IsAnimValList());
 
-  Element()->DidChangeLengthList(AttrEnum(), emptyOrOldValue);
-  if (mAList->IsAnimating()) {
-    Element()->AnimationNeedsResample();
-  }
   return domItem.forget();
 }
 
-already_AddRefed<nsIDOMSVGLength>
+already_AddRefed<DOMSVGLength>
 DOMSVGLengthList::RemoveItem(uint32_t index,
                              ErrorResult& error)
 {
@@ -301,38 +331,37 @@ DOMSVGLengthList::RemoveItem(uint32_t index,
     return nullptr;
   }
 
-  nsAttrValue emptyOrOldValue = Element()->WillChangeLengthList(AttrEnum());
+  AutoChangeLengthListNotifier notifier(this);
   // Now that we know we're removing, keep animVal list in sync as necessary.
   // Do this *before* touching InternalList() so the removed item can get its
   // internal value.
   MaybeRemoveItemFromAnimValListAt(index);
 
-  // We have to return the removed item, so make sure it exists:
-  EnsureItemAt(index);
+  // We have to return the removed item, so get it, creating it if necessary:
+  nsCOMPtr<DOMSVGLength> result = GetItemAt(index);
 
   // Notify the DOM item of removal *before* modifying the lists so that the
   // DOM item can copy its *old* value:
   mItems[index]->RemovingFromList();
-  nsCOMPtr<nsIDOMSVGLength> result = mItems[index];
 
   InternalList().RemoveItem(index);
   mItems.RemoveElementAt(index);
 
   UpdateListIndicesFromIndex(mItems, index);
 
-  Element()->DidChangeLengthList(AttrEnum(), emptyOrOldValue);
-  if (mAList->IsAnimating()) {
-    Element()->AnimationNeedsResample();
-  }
   return result.forget();
 }
 
-void
-DOMSVGLengthList::EnsureItemAt(uint32_t aIndex)
+already_AddRefed<DOMSVGLength>
+DOMSVGLengthList::GetItemAt(uint32_t aIndex)
 {
+  MOZ_ASSERT(aIndex < mItems.Length());
+
   if (!mItems[aIndex]) {
     mItems[aIndex] = new DOMSVGLength(this, AttrEnum(), aIndex, IsAnimValList());
   }
+  nsRefPtr<DOMSVGLength> result = mItems[aIndex];
+  return result.forget();
 }
 
 void
