@@ -5,7 +5,7 @@
 
 
 /**
-   File Name:          7.4.3-1-n.js
+   File Name:          7.4.3-8-n.js
    ECMA Section:       7.4.3
 
    Description:
@@ -22,16 +22,16 @@
    Author:             christine@netscape.com
    Date:               12 november 1997
 */
-var SECTION = "7.4.3-1-n";
+var SECTION = "7.4.3-9-n";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "Future Reserved Words";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-DESCRIPTION = "var case = true";
+DESCRIPTION = "var switch = true";
 EXPECTED = "error";
 
-new TestCase( SECTION,  "var case = true",     "error",    eval("var case = true") );
+new TestCase( SECTION,  "var switch = true",     "error",    eval("var switch = true") );
 
 test();
