@@ -1,36 +1,27 @@
-// |reftest| skip-if(Android)
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 465567;
+var summary = 'TM: Do not assert: JSVAL_TAG(v) == JSVAL_OBJECT';
 var actual = '';
 var expect = '';
 
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
+jit(true);
 
-function test()
+try
 {
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
-
-// ------- Comment #135 From Gary Kwong [:nth10sd]
-
-  if (typeof timeout == 'function')
-  {
-    expectExitCode(6);
-    timeout(3);
-    eval("do ([]); while(y for each (x in []))");
-  }
-
-  reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
+  eval("for each (e in ['', true, 1, true, 1]) { e = null; if (0) { let e; var e; } }");
 }
+catch(ex)
+{
+}
+
+jit(false);
+
+reportCompare(expect, actual, summary);
