@@ -4,15 +4,20 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch07/7.6/7.6.1/7.6.1.2/7.6.1.2-14-s.js
- * @description Strict Mode - SyntaxError isn't thrown when 'implements0' occurs in strict mode code
- * @onlyStrict
+ * @path ch07/7.6/7.6.1/7.6.1.2/7.6.1-18-s.js
+ * @description 7.6 - SyntaxError expected: reserved words used as Identifier Names in UTF8: l\u0065t (let)
+ * @onlyStrict 
  */
 
 
-function testcase() {
+function testcase() {        
         "use strict";
-        var implements0 = 1;
-        return implements0 === 1;
-    }
-runTestCase(testcase);
+
+        try {
+            eval("var l\u0065t = 123;");
+            return false;
+        } catch (e) {
+            return e instanceof SyntaxError;
+        }
+}
+runTestCase(testcase);
\ No newline at end of file
