@@ -4,15 +4,13 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.1/10.1.1/10.1.1-7-s.js
- * @description Strict Mode - Use Strict Directive Prologue is ''use strict';' which appears at the end of the block
- * @noStrict
+ * @path ch10/10.4/10.4.3/10.4.3-1-17-s.js
+ * @description Strict Mode - checking 'this' (eval used within strict mode)
+ * @onlyStrict
  */
-
-
+    
 function testcase() {
-        var public = 1;
-        return public === 1;
-        "use strict";
-    }
-runTestCase(testcase);
+"use strict";
+return (eval("typeof this") === "undefined") && (eval("this") !== fnGlobalObject());
+}
+runTestCase(testcase);
\ No newline at end of file
