@@ -4,13 +4,12 @@
 /// "Use Terms").   Any redistribution of this code must retain the above
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-15gs.js
- * @description Strict - checking 'this' from a global scope (New'ed Function constructor defined within strict mode)
+ * @path ch10/10.4/10.4.3/10.4.3-1-66gs.js
+ * @description Strict - checking 'this' from a global scope (strict function declaration called by Function.prototype.apply())
  * @onlyStrict
  */
 
-"use strict";
-var f = new Function("return typeof this;");
-if (f() === "undefined") {
+function f() { "use strict"; return this===undefined;};
+if (! f.apply()){
     throw "'this' had incorrect value!";
-}
+}
\ No newline at end of file
