@@ -5,8 +5,8 @@
  * No matter how control leaves the embedded 'Statement',
  * the scope chain is always restored to its former state
  *
- * @path ch12/12.10/S12.10_A3.1_T2.js
- * @description Using "with" statement within global context, leading to completion by exception
+ * @path ch12/12.10/S12.10_A3.8_T2.js
+ * @description Declaring function constructor within "with" statement, leading to normal completion by "return"
  * @noStrict
  */
 
@@ -20,39 +20,26 @@ var myObj = {
     valueOf : function(){return 'obj_valueOf';}
 }
 
-try {
-
-    with(myObj){
-  
-        p1 = 'x1'
-        throw value;
-    
+with(myObj){
+    var __FACTORY = function(){
+        p1 = 'x1';
+        return value;
     }
-} catch(e){
-    result = p1;
+    var obj = new __FACTORY;
 }
 
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#1
-if(result !== 1){
-    $ERROR('#1: result === 1. Actual:  result ==='+ result  );
+if(p1 !== 1){
+  $ERROR('#1: p1 === 1. Actual:  p1 ==='+ p1  );
 }
 //
 //////////////////////////////////////////////////////////////////////////////
 
-
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#2
-if(p1 !== 1){
-  $ERROR('#2: p1 === 1. Actual:  p1 ==='+ p1  );
-}
-//
-//////////////////////////////////////////////////////////////////////////////
-
-//////////////////////////////////////////////////////////////////////////////
-//CHECK#3
-if (myObj.p1 !== "x1") {
-	$ERROR('#3: myObj.p1 === "x1". Actual:  myObj.p1 ==='+ myObj.p1  );
+if(myObj.p1 !== "x1"){
+  $ERROR('#2: myObj.p1 === "x1". Actual:  myObj.p1 ==='+ myObj.p1  );
 }
 //
 //////////////////////////////////////////////////////////////////////////////
