@@ -7,18 +7,17 @@
  * Refer 13; 
  * The production FunctionBody : SourceElementsopt is evaluated as follows:
  *
- * @path ch13/13.0/13.0-8-s.js
- * @description Strict Mode - SourceElements is evaluated as strict mode code when the code of this FunctionExpression is contained in non-strict mode but the call to eval is a direct call in strict mode code
+ * @path ch13/13.0/13.0-14-s.js
+ * @description Strict Mode - SourceElements is evaluated as strict mode code when the function body of a Function constructor contains a Strict Directive
  * @onlyStrict
  */
 
 
 function testcase() {
-        "use strict";
 
         try {
-            eval("var _13_0_8_fun = function () {eval = 42;};");
-            _13_0_8_fun();
+            var _13_0_14_fun = new Function(" ", "'use strict'; eval = 42; ");
+            _13_0_14_fun();
             return false;
         } catch (e) {
             return e instanceof SyntaxError;
